@@ -1,7 +1,6 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
 # -*- coding: utf-8 -*-
 
-from __future__ import absolute_import, division, unicode_literals, print_function
 
 import os
 import sys
@@ -55,8 +54,6 @@ def test_defragment_bzp2(tmpdir):
     _test_defragment(tmpdir, 'bzp2')
 
 
-@pytest.mark.skipif(sys.platform.startswith('win'),
-    reason="This test fails on AppVeyor even though the lz4 module exists")
 def test_defragment_lz4(tmpdir):
     pytest.importorskip('lz4')
     _test_defragment(tmpdir, 'lz4')
