@@ -1,23 +1,18 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
 # -*- coding: utf-8 -*-
 
-from __future__ import absolute_import, division, unicode_literals, print_function
-
-import datetime
-import json
 import os
-
+import json
+import datetime
+from functools import lru_cache
 from collections import OrderedDict
-
-import six
-from six.moves.urllib import parse as urlparse
+from urllib import parse as urlparse
 
 from jsonschema import validators as mvalidators
 from jsonschema.exceptions import ValidationError
 
 import yaml
 
-from .compat import lru_cache
 from . import constants
 from . import generic_io
 from . import reference
@@ -38,13 +33,9 @@ else:  # pragma: no cover
 __all__ = ['validate', 'fill_defaults', 'remove_defaults', 'check_schema']
 
 
-SCHEMA_PATH = os.path.abspath(
-    os.path.join(os.path.dirname(__file__), 'schemas'))
-
-
 PYTHON_TYPE_TO_YAML_TAG = {
     None: 'null',
-    six.text_type: 'str',
+    str: 'str',
     bytes: 'str',
     bool: 'bool',
     int: 'int',
@@ -56,10 +47,6 @@ PYTHON_TYPE_TO_YAML_TAG = {
 }
 
 
-if six.PY2: # pragma: no cover
-    PYTHON_TYPE_TO_YAML_TAG[long] = 'int'
-
-
 # Prepend full YAML tag prefix
 for k, v in PYTHON_TYPE_TO_YAML_TAG.items():
     PYTHON_TYPE_TO_YAML_TAG[k] = constants.YAML_TAG_PREFIX + v
@@ -69,14 +56,10 @@ def _type_to_tag(type_):
     for base in type_.mro():
         if base in PYTHON_TYPE_TO_YAML_TAG:
             return PYTHON_TYPE_TO_YAML_TAG[base]
+    return None
 
 
 def validate_tag(validator, tagname, instance, schema):
-    # Shortcut: If the instance is a subclass of YAMLObject then we know it
-    # should have a yaml_tag attribute attached; otherwise we have to use a
-    # hack of reserializing the object and seeing what tags get attached to it
-    # (though there may be a better way than this).
-
     if hasattr(instance, '_tag'):
         instance_tag = instance._tag
     else:
@@ -167,7 +150,7 @@ def validate_fill_default(validator, properties, instance, schema):
     if not validator.is_type(instance, 'object'):
         return
 
-    for property, subschema in six.iteritems(properties):
+    for property, subschema in properties.items():
         if "default" in subschema:
             instance.setdefault(property, subschema["default"])
 
@@ -186,7 +169,7 @@ def validate_remove_default(validator, properties, instance, schema):
     if not validator.is_type(instance, 'object'):
         return
 
-    for property, subschema in six.iteritems(properties):
+    for property, subschema in properties.items():
         if subschema.get("default", None) is not None:
             if instance.get(property, None) == subschema["default"]:
                 del instance[property]
@@ -205,10 +188,8 @@ REMOVE_DEFAULTS['properties'] = validate_remove_default
 
 @lru_cache()
 def _create_validator(validators=YAML_VALIDATORS):
-    meta_schema = load_schema(YAML_SCHEMA_METASCHEMA_ID,
-                              mresolver.default_url_mapping)
-    base_cls = mvalidators.create(meta_schema=meta_schema,
-                                  validators=validators)
+    meta_schema = load_schema(YAML_SCHEMA_METASCHEMA_ID, mresolver.default_resolver)
+    base_cls = mvalidators.create(meta_schema=meta_schema, validators=validators)
 
     class ASDFValidator(base_cls):
         DEFAULT_TYPES = base_cls.DEFAULT_TYPES.copy()
@@ -234,9 +215,9 @@ def _create_validator(validators=YAML_VALIDATORS):
             if _schema is None:
                 tag = getattr(instance, '_tag', None)
                 if tag is not None:
-                    schema_path = self.ctx.tag_to_schema_resolver(tag)
+                    schema_path = self.ctx.resolver(tag)
                     if schema_path != tag:
-                        s = load_schema(schema_path, self.ctx.url_mapping)
+                        s = load_schema(schema_path, self.ctx.resolver)
                         if s:
                             with self.resolver.in_scope(schema_path):
                                 for x in super(ASDFValidator, self).iter_errors(instance, s):
@@ -244,7 +225,7 @@ def _create_validator(validators=YAML_VALIDATORS):
 
                 if isinstance(instance, dict):
                     new_seen = _seen | set([id(instance)])
-                    for val in six.itervalues(instance):
+                    for val in instance.values():
                         for x in self.iter_errors(val, _seen=new_seen):
                             yield x
 
@@ -275,16 +256,10 @@ OrderedLoader.add_constructor(
     construct_mapping)
 
 
-if six.PY2: # pragma: no cover
-    # Load strings in as Unicode on Python 2
-    OrderedLoader.add_constructor('tag:yaml.org,2002:str',
-                                  OrderedLoader.construct_scalar)
-
-
 @lru_cache()
 def _load_schema(url):
     with generic_io.get_file(url) as fd:
-        if isinstance(url, six.text_type) and url.endswith('json'):
+        if isinstance(url, str) and url.endswith('json'):
             json_data = fd.read().decode('utf-8')
             result = json.loads(json_data, object_pairs_hook=OrderedDict)
         else:
@@ -306,7 +281,7 @@ def _make_resolver(url_mapping):
     def get_schema(url):
         return schema_loader(url)[0]
 
-    for x in ['http', 'https', 'file']:
+    for x in ['http', 'https', 'file', 'tag']:
         handlers[x] = get_schema
 
     # We set cache_remote=False here because we do the caching of
@@ -330,7 +305,29 @@ HARDCODED_SCHEMA = {
 
 
 @lru_cache()
-def load_schema(url, resolver=None, resolve_references=False):
+def load_custom_schema(url):
+    # Avoid circular import
+    from .tags.core import AsdfObject
+    custom = load_schema(url, resolve_local_refs=True)
+    core = load_schema(AsdfObject.yaml_tag)
+
+    def update(d, u):
+        from collections import Mapping
+        for k, v in u.items():
+            # Respect the property ordering of the core schema
+            if k == 'propertyOrder' and k in d:
+                d[k] = u[k] + d[k]
+            elif isinstance(v, Mapping):
+                d[k] = update(d.get(k, {}), v)
+            else:
+                d[k] = v
+        return d
+
+    return update(custom, core)
+
+@lru_cache()
+def load_schema(url, resolver=None, resolve_references=False,
+                resolve_local_refs=False):
     """
     Load a schema from the given URL.
 
@@ -347,15 +344,35 @@ def load_schema(url, resolver=None, resolve_references=False):
 
     resolve_references : bool, optional
         If `True`, resolve all `$ref` references.
+
+    resolve_local_refs : bool, optional
+        If `True`, resolve all `$ref` references that refer to other objects
+        within the same schema. This will automatically be handled when passing
+        `resolve_references=True`, but it may be desirable in some cases to
+        control local reference resolution separately.
     """
     if resolver is None:
-        resolver = mresolver.default_url_mapping
+        resolver = mresolver.default_resolver
     loader = _make_schema_loader(resolver)
     if url in HARDCODED_SCHEMA:
         schema = HARDCODED_SCHEMA[url]()
     else:
         schema, url = loader(url)
 
+    # Resolve local references
+    if resolve_local_refs:
+        def resolve_local(node, json_id):
+            if isinstance(node, dict) and '$ref' in node:
+                ref_url = resolver(node['$ref'])
+                if ref_url.startswith('#'):
+                    parts = urlparse.urlparse(ref_url)
+                    subschema_fragment = reference.resolve_fragment(
+                        schema, parts.fragment)
+                    return subschema_fragment
+            return node
+
+        schema = treeutil.walk_and_modify(schema, resolve_local)
+
     if resolve_references:
         def resolve_refs(node, json_id):
             if json_id is None:
@@ -373,10 +390,13 @@ def load_schema(url, resolver=None, resolve_references=False):
                     subschema = schema
                 else:
                     subschema = load_schema(suburl_path, resolver, True)
+
                 subschema_fragment = reference.resolve_fragment(
                     subschema, fragment)
                 return subschema_fragment
+
             return node
+
         schema = treeutil.walk_and_modify(schema, resolve_refs)
 
     return schema
@@ -431,39 +451,21 @@ def get_validator(schema={}, ctx=None, validators=None, url_mapping=None,
     return validator
 
 
-if six.PY2: # pragma: no cover
-    def validate_large_literals(instance):
-        """
-        Validate that the tree has no large numeric literals.
-        """
-        # We can count on 52 bits of precision
-        upper = ((long(1) << 51) - 1)
-        lower = -((long(1) << 51) - 2)
-
-        for instance in treeutil.iter_tree(instance):
-            if (isinstance(instance, six.integer_types) and
-                (instance > upper or instance < lower)):
-                raise ValidationError(
-                    "Integer value {0} is too large to safely represent as a "
-                    "literal in ASDF".format(instance))
-else:
-    def validate_large_literals(instance):
-        """
-        Validate that the tree has no large numeric literals.
-        """
-        # We can count on 52 bits of precision
-        for instance in treeutil.iter_tree(instance):
-            if (isinstance(instance, int) and (
-                instance > ((1 << 51) - 1) or
-                instance < -((1 << 51) - 2))):
-                raise ValidationError(
-                    "Integer value {0} is too large to safely represent as a "
-                    "literal in ASDF".format(instance))
+def validate_large_literals(instance):
+    """
+    Validate that the tree has no large numeric literals.
+    """
+    # We can count on 52 bits of precision
+    for instance in treeutil.iter_tree(instance):
+        if (isinstance(instance, int) and (
+            instance > ((1 << 51) - 1) or
+            instance < -((1 << 51) - 2))):
+            raise ValidationError(
+                "Integer value {0} is too large to safely represent as a "
+                "literal in ASDF".format(instance))
 
 
-def validate(instance, ctx=None, schema={},
-             validators=None,
-             *args, **kwargs):
+def validate(instance, ctx=None, schema={}, validators=None, *args, **kwargs):
     """
     Validate the given instance (which must be a tagged tree) against
     the appropriate schema.  The schema itself is located using the
@@ -491,7 +493,7 @@ def validate(instance, ctx=None, schema={},
         from .asdf import AsdfFile
         ctx = AsdfFile()
 
-    validator = get_validator(schema, ctx, validators, ctx.url_mapping,
+    validator = get_validator(schema, ctx, validators, ctx.resolver,
                               *args, **kwargs)
     validator.validate(instance, _schema=(schema or None))
 
@@ -552,10 +554,9 @@ def check_schema(schema):
         'default': validate_default
     })
 
-    meta_schema = load_schema(YAML_SCHEMA_METASCHEMA_ID,
-                              mresolver.default_url_mapping)
+    meta_schema = load_schema(YAML_SCHEMA_METASCHEMA_ID, mresolver.default_resolver)
 
-    resolver = _make_resolver(mresolver.default_url_mapping)
+    resolver = _make_resolver(mresolver.default_resolver)
 
     cls = mvalidators.create(meta_schema=meta_schema,
                              validators=VALIDATORS)
