@@ -1,10 +1,6 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
 # -*- coding: utf-8 -*-
 
-from __future__ import absolute_import, division, unicode_literals, print_function
-
-import six
-
 import numpy as np
 
 from ...asdftypes import AsdfType
@@ -17,7 +13,7 @@ class ComplexType(AsdfType):
 
     @classmethod
     def to_tree(cls, node, ctx):
-        return six.text_type(node)
+        return str(node)
 
     @classmethod
     def from_tree(cls, tree, ctx):
