@@ -1,8 +1,6 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
 # -*- coding: utf-8 -*-
 
-from __future__ import absolute_import, division, unicode_literals, print_function
-
 import sys
 
 import numpy as np
@@ -10,8 +8,6 @@ from numpy import ma
 
 from jsonschema import ValidationError
 
-import six
-
 from ...asdftypes import AsdfType
 from ... import schema
 from ... import util
@@ -52,14 +48,14 @@ def asdf_byteorder_to_numpy_byteorder(byteorder):
 def asdf_datatype_to_numpy_dtype(datatype, byteorder=None):
     if byteorder is None:
         byteorder = sys.byteorder
-    if isinstance(datatype, six.string_types) and datatype in _datatype_names:
+    if isinstance(datatype, str) and datatype in _datatype_names:
         datatype = _datatype_names[datatype]
         byteorder = asdf_byteorder_to_numpy_byteorder(byteorder)
         return np.dtype(str(byteorder + datatype))
     elif (isinstance(datatype, list) and
           len(datatype) == 2 and
-          isinstance(datatype[0], six.text_type) and
-          isinstance(datatype[1], six.integer_types) and
+          isinstance(datatype[0], str) and
+          isinstance(datatype[1], int) and
           datatype[0] in _string_datatype_names):
         length = datatype[1]
         byteorder = asdf_byteorder_to_numpy_byteorder(byteorder)
@@ -390,14 +386,19 @@ class NDArrayType(AsdfType):
     @classmethod
     def to_tree(cls, data, ctx):
         base = util.get_array_base(data)
-        block = ctx.blocks.find_or_create_block_for_array(data, ctx)
         shape = data.shape
         dtype = data.dtype
         offset = data.ctypes.data - base.ctypes.data
-        if data.flags[b'C_CONTIGUOUS']:
-            strides = None
-        else:
-            strides = data.strides
+        strides = None
+
+        if not data.flags.c_contiguous:
+            # We do not want to encode strides for broadcasted arrays
+            if not all(data.strides):
+                data = np.ascontiguousarray(data)
+            else:
+                strides = data.strides
+
+        block = ctx.blocks.find_or_create_block_for_array(data, ctx)
 
         result = {}
 
