@@ -1,24 +1,22 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
 # -*- coding: utf-8 -*-
 
-from __future__ import absolute_import, division, unicode_literals, print_function
 
 import io
 import os
 import sys
 import pytest
 
-from .. import asdf
-from .. import asdftypes
-from .. import extension
-from .. import util
-from .. import versioning
+import asdf
+from asdf import asdftypes
+from asdf import extension
+from asdf import util
+from asdf import versioning
 
 from . import helpers, CustomTestType
-from astropy.tests.helper import catch_warnings
 
 
-TEST_DATA_PATH = os.path.join(os.path.dirname(__file__), 'data')
+TEST_DATA_PATH = str(helpers.get_test_data_path(''))
 
 
 def test_custom_tag():
@@ -84,9 +82,9 @@ b: !core/complex-1.0.0
             buff, extensions=FractionCallable()) as ff:
         assert ff.tree['a'] == fractions.Fraction(2, 3)
 
-    buff = io.BytesIO()
-    ff.write_to(buff)
-    buff.close()
+        buff = io.BytesIO()
+        ff.write_to(buff)
+        buff.close()
 
 
 def test_version_mismatch():
@@ -96,7 +94,7 @@ a: !core/complex-42.0.0
     """
 
     buff = helpers.yaml_to_asdf(yaml)
-    with catch_warnings() as warning:
+    with pytest.warns(None) as warning:
         with asdf.AsdfFile.open(buff, ignore_version_mismatch=False) as ff:
             assert isinstance(ff.tree['a'], complex)
 
@@ -107,7 +105,7 @@ a: !core/complex-42.0.0
 
     # Make sure warning is repeatable
     buff.seek(0)
-    with catch_warnings() as warning:
+    with pytest.warns(None) as warning:
         with asdf.AsdfFile.open(buff, ignore_version_mismatch=False) as ff:
             assert isinstance(ff.tree['a'], complex)
 
@@ -118,7 +116,7 @@ a: !core/complex-42.0.0
 
     # Make sure the warning does not occur if it is being ignored (default)
     buff.seek(0)
-    with catch_warnings() as warning:
+    with pytest.warns(None) as warning:
         with asdf.AsdfFile.open(buff) as ff:
             assert isinstance(ff.tree['a'], complex)
 
@@ -132,7 +130,7 @@ a: !core/complex-1.0.1
     """
 
     buff = helpers.yaml_to_asdf(yaml)
-    with catch_warnings() as warning:
+    with pytest.warns(None) as warning:
         with asdf.AsdfFile.open(buff, ignore_version_mismatch=False) as ff:
             assert isinstance(ff.tree['a'], complex)
 
@@ -152,7 +150,7 @@ a: !core/complex-42.0.0
     with open(testfile, 'wb') as handle:
         handle.write(buff.read())
 
-    with catch_warnings() as w:
+    with pytest.warns(None) as w:
         with asdf.AsdfFile.open(testfile, ignore_version_mismatch=False) as ff:
             assert ff._fname == "file://{}".format(testfile)
             assert isinstance(ff.tree['a'], complex)
@@ -201,7 +199,7 @@ flow_thing:
     d: 3.14
 """
     buff = helpers.yaml_to_asdf(yaml)
-    with catch_warnings() as w:
+    with pytest.warns(None) as w:
         data = asdf.AsdfFile.open(
             buff, ignore_version_mismatch=False,
             extensions=CustomFlowExtension())
@@ -226,24 +224,13 @@ def test_versioned_writing():
 
     versioning.supported_versions.append(versioning.AsdfVersion('42.0.0'))
 
-    # Currently this class cannot inherit directly from ComplexType because if
-    # it does it pollutes ASDF's built-in extension and causes later tests that
-    # rely on ComplexType to fail. However, if CustomType is ever implemented
-    # as an abstract base class, then it will be possible to use it as a mix-in
-    # and also inherit from ComplexType. This means the only method/attribute
-    # that will need to be explicitly defined will be 'version'.
-    class FancyComplexType(asdftypes.CustomType):
-        name = ComplexType.name
+    class FancyComplexType(ComplexType, asdftypes.CustomType):
         version = (42, 0, 0)
-        types = ComplexType.types
 
-        @classmethod
-        def to_tree(cls, node, ctx):
-            return ComplexType.to_tree(node, ctx)
-
-        @classmethod
-        def from_tree(cls, tree, ctx):
-            return ComplexType.from_tree(tree, ctx)
+    # This is a sanity check to ensure that the custom FancyComplexType does
+    # not get added to ASDF's built-in extension, since this would cause any
+    # subsequent tests that rely on ComplexType to fail.
+    assert not issubclass(FancyComplexType, asdftypes.AsdfTypeMeta)
 
     class FancyComplexExtension(object):
         @property
@@ -337,7 +324,7 @@ undefined_data:
         - !core/complex-1.0.0 3.14j
 """
     buff = helpers.yaml_to_asdf(yaml)
-    with catch_warnings() as warning:
+    with pytest.warns(None) as warning:
         afile = asdf.AsdfFile.open(buff)
         missing = afile.tree['undefined_data']
 
@@ -356,7 +343,7 @@ undefined_data:
 
     # Make sure no warning occurs if explicitly ignored
     buff.seek(0)
-    with catch_warnings() as warning:
+    with pytest.warns(None) as warning:
         afile = asdf.AsdfFile.open(buff, ignore_unrecognized_tag=True)
     assert len(warning) == 0
 
@@ -425,7 +412,7 @@ flow_thing:
     b: 3.14
 """
     old_buff = helpers.yaml_to_asdf(old_yaml)
-    with catch_warnings() as warning:
+    with pytest.warns(None) as warning:
         asdf.AsdfFile.open(old_buff, extensions=CustomFlowExtension())
 
     assert len(warning) == 1, helpers.display_warnings(warning)
@@ -591,7 +578,7 @@ flow_thing:
 """
     buff = helpers.yaml_to_asdf(yaml)
 
-    with catch_warnings() as _warnings:
+    with pytest.warns(None) as _warnings:
         data = asdf.AsdfFile.open(buff, extensions=CustomFlowExtension())
 
     assert len(_warnings) == 1
