@@ -1,7 +1,6 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
 # -*- coding: utf-8 -*-
 
-from __future__ import absolute_import, division, unicode_literals, print_function
 
 import copy
 import os
@@ -13,15 +12,13 @@ from numpy.testing import assert_array_equal
 
 astropy = pytest.importorskip('astropy')
 from astropy.io import fits
-from astropy.tests.helper import catch_warnings
+from astropy.table import Table
 
-from .. import asdf
-from .. import fits_embed
-from .. import open as asdf_open
-from .helpers import assert_tree_match, yaml_to_asdf, display_warnings
+import asdf
+from asdf import fits_embed
+from asdf import open as asdf_open
 
-
-TEST_DATA_PATH = os.path.join(os.path.dirname(__file__), 'data')
+from .helpers import assert_tree_match, display_warnings, get_test_data_path
 
 
 def create_asdf_in_fits():
@@ -50,7 +47,13 @@ def create_asdf_in_fits():
 
     return fits_embed.AsdfInFits(hdulist, tree)
 
-def test_embed_asdf_in_fits_file(tmpdir):
+# Testing backwards compatibility ensures that we can continue to read and
+# write files that use the old convention of ImageHDU to store the ASDF file.
+@pytest.mark.parametrize('backwards_compat', [False, True])
+def test_embed_asdf_in_fits_file(tmpdir, backwards_compat):
+    fits_testfile = str(tmpdir.join('test.fits'))
+    asdf_testfile = str(tmpdir.join('test.asdf'))
+
     hdulist = fits.HDUList()
     hdulist.append(fits.ImageHDU(np.arange(512, dtype=np.float), name='SCI'))
     hdulist.append(fits.ImageHDU(np.arange(512, dtype=np.float), name='DQ'))
@@ -69,24 +72,29 @@ def test_embed_asdf_in_fits_file(tmpdir):
     }
 
     ff = fits_embed.AsdfInFits(hdulist, tree)
-    ff.write_to(os.path.join(str(tmpdir), 'test.fits'))
+    ff.write_to(fits_testfile, use_image_hdu=backwards_compat)
 
-    ff2 = asdf.AsdfFile(tree)
-    ff2.write_to(os.path.join(str(tmpdir), 'plain.asdf'))
-
-    with fits.open(os.path.join(str(tmpdir), 'test.fits')) as hdulist2:
+    with fits.open(fits_testfile) as hdulist2:
         assert len(hdulist2) == 3
         assert [x.name for x in hdulist2] == ['SCI', 'DQ', 'ASDF']
         assert_array_equal(hdulist2[0].data, np.arange(512, dtype=np.float))
-        assert hdulist2['ASDF'].data.tostring().strip().endswith(b"...")
+        asdf_hdu = hdulist2['ASDF']
+        assert asdf_hdu.data.tostring().startswith(b'#ASDF')
+        # When in backwards compatibility mode, the ASDF file will be contained
+        # in an ImageHDU
+        if backwards_compat:
+            assert isinstance(asdf_hdu, fits.ImageHDU)
+            assert asdf_hdu.data.tostring().strip().endswith(b'...')
+        else:
+            assert isinstance(asdf_hdu, fits_embed._AsdfHDU)
 
         with fits_embed.AsdfInFits.open(hdulist2) as ff2:
             assert_tree_match(tree, ff2.tree)
 
             ff = asdf.AsdfFile(copy.deepcopy(ff2.tree))
-            ff.write_to('test.asdf')
+            ff.write_to(asdf_testfile)
 
-    with asdf.AsdfFile.open('test.asdf') as ff:
+    with asdf.AsdfFile.open(asdf_testfile) as ff:
         assert_tree_match(tree, ff.tree)
 
 
@@ -101,18 +109,74 @@ def test_embed_asdf_in_fits_file_anonymous_extensions(tmpdir):
     with fits.open(os.path.join(str(tmpdir), 'test.fits')) as hdulist:
         assert len(hdulist) == 4
         assert [x.name for x in hdulist] == ['PRIMARY', '', '', 'ASDF']
-        assert hdulist['ASDF'].data.tostring().strip().endswith(b"...")
+        asdf_hdu = hdulist['ASDF']
+        assert isinstance(asdf_hdu, fits_embed._AsdfHDU)
+        assert asdf_hdu.data.tostring().startswith(b'#ASDF')
 
         with fits_embed.AsdfInFits.open(hdulist) as ff2:
             assert_tree_match(asdf_in_fits.tree, ff2.tree)
 
             ff = asdf.AsdfFile(copy.deepcopy(ff2.tree))
-            ff.write_to('test.asdf')
+            ff.write_to(os.path.join(str(tmpdir), 'test.asdf'))
 
-    with asdf.AsdfFile.open('test.asdf') as ff:
+    with asdf.AsdfFile.open(os.path.join(str(tmpdir), 'test.asdf')) as ff:
         assert_tree_match(asdf_in_fits.tree, ff.tree)
 
 
+@pytest.mark.xfail(
+    reason="In-place update for ASDF-in-FITS does not currently work")
+def test_update_in_place(tmpdir):
+    tempfile = str(tmpdir.join('test.fits'))
+
+    # Create a file and write it out
+    asdf_in_fits = create_asdf_in_fits()
+    asdf_in_fits.write_to(tempfile)
+
+    # Open the file and add data so it needs to be updated
+    with fits_embed.AsdfInFits.open(tempfile) as ff:
+        ff.tree['new_stuff'] = "A String"
+        ff.update()
+
+    # Open the updated file and make sure everything looks okay
+    with fits_embed.AsdfInFits.open(tempfile) as ff:
+        assert ff.tree['new_stuff'] == "A String"
+        assert_tree_match(ff.tree['model'], asdf_in_fits.tree['model'])
+
+
+def test_update_and_write_new(tmpdir):
+    tempfile = str(tmpdir.join('test.fits'))
+    newfile = str(tmpdir.join('new.fits'))
+
+    # Create a file and write it out
+    asdf_in_fits = create_asdf_in_fits()
+    asdf_in_fits.write_to(tempfile)
+
+    # Open the file and add data so it needs to be updated
+    with fits_embed.AsdfInFits.open(tempfile) as ff:
+        ff.tree['new_stuff'] = "A String"
+        ff.write_to(newfile)
+
+    # Open the updated file and make sure everything looks okay
+    with fits_embed.AsdfInFits.open(newfile) as ff:
+        assert ff.tree['new_stuff'] == "A String"
+        assert_tree_match(ff.tree['model'], asdf_in_fits.tree['model'])
+
+
+@pytest.mark.xfail(
+    reason="ASDF HDU implementation does not currently reseek after writing")
+def test_access_hdu_data_after_write(tmpdir):
+    # There is actually probably not a great reason to support this kind of
+    # functionality, but I am adding a test here to record the failure for
+    # posterity.
+    tempfile = str(tmpdir.join('test.fits'))
+
+    asdf_in_fits = create_asdf_in_fits()
+    asdf_in_fits.write_to(tempfile)
+    asdf_hdu = asdf_in_fits._hdulist['ASDF']
+
+    assert asdf_hdu.data.tostring().startswith('#ASDF')
+
+
 def test_create_in_tree_first(tmpdir):
     tree = {
         'model': {
@@ -214,6 +278,20 @@ def test_asdf_open(tmpdir):
         with asdf_open(hdulist) as ff:
             compare_asdfs(asdf_in_fits, ff)
 
+def test_open_gzipped():
+    testfile = get_test_data_path('asdf.fits.gz')
+
+    # Opening as an HDU should work
+    with fits.open(testfile) as ff:
+        with asdf.AsdfFile.open(ff) as af:
+            assert af.tree['stuff'].shape == (20, 20)
+
+    with fits_embed.AsdfInFits.open(testfile) as af:
+        assert af.tree['stuff'].shape == (20, 20)
+
+    with asdf.AsdfFile.open(testfile) as af:
+        assert af.tree['stuff'].shape == (20, 20)
+
 def test_bad_input(tmpdir):
     """Make sure these functions behave properly with bad input"""
     text_file = os.path.join(str(tmpdir), 'test.txt')
@@ -227,9 +305,9 @@ def test_bad_input(tmpdir):
 @pytest.mark.skipif(sys.platform.startswith('win'),
     reason='Avoid path manipulation on Windows')
 def test_version_mismatch_file():
-    testfile = os.path.join(TEST_DATA_PATH, 'version_mismatch.fits')
+    testfile = str(get_test_data_path('version_mismatch.fits'))
 
-    with catch_warnings() as w:
+    with pytest.warns(None) as w:
         with asdf.AsdfFile.open(testfile,
                 ignore_version_mismatch=False) as fits_handle:
             assert fits_handle.tree['a'] == complex(0j)
@@ -237,25 +315,65 @@ def test_version_mismatch_file():
     assert len(w) == 1
     assert str(w[0].message) == (
         "'tag:stsci.edu:asdf/core/complex' with version 7.0.0 found in file "
-        "'file://{}', but latest supported version is 1.0.0".format(testfile))
+        "'{}', but latest supported version is 1.0.0".format(testfile))
 
     # Make sure warning does not occur when warning is ignored (default)
-    with catch_warnings() as w:
+    with pytest.warns(None) as w:
         with asdf.AsdfFile.open(testfile) as fits_handle:
             assert fits_handle.tree['a'] == complex(0j)
     assert len(w) == 0, display_warnings(w)
 
-    with catch_warnings() as w:
+    with pytest.warns(None) as w:
         with fits_embed.AsdfInFits.open(testfile,
                 ignore_version_mismatch=False) as fits_handle:
             assert fits_handle.tree['a'] == complex(0j)
     assert len(w) == 1
     assert str(w[0].message) == (
         "'tag:stsci.edu:asdf/core/complex' with version 7.0.0 found in file "
-        "'file://{}', but latest supported version is 1.0.0".format(testfile))
+        "'{}', but latest supported version is 1.0.0".format(testfile))
 
     # Make sure warning does not occur when warning is ignored (default)
-    with catch_warnings() as w:
+    with pytest.warns(None) as w:
         with fits_embed.AsdfInFits.open(testfile) as fits_handle:
             assert fits_handle.tree['a'] == complex(0j)
     assert len(w) == 0, display_warnings(w)
+
+def test_serialize_table(tmpdir):
+    tmpfile = str(tmpdir.join('table.fits'))
+
+    data = np.random.random((10, 10))
+    table = Table(data)
+
+    hdu = fits.BinTableHDU(table)
+    hdulist = fits.HDUList()
+    hdulist.append(hdu)
+
+    tree = {'my_table': hdulist[1].data}
+    with fits_embed.AsdfInFits(hdulist, tree) as ff:
+        ff.write_to(tmpfile)
+
+    with asdf.AsdfFile.open(tmpfile) as ff:
+        data = ff.tree['my_table']
+        assert data._source.startswith('fits:')
+
+def test_extension_check():
+    testfile = get_test_data_path('extension_check.fits')
+
+    with pytest.warns(None) as warnings:
+        with asdf.AsdfFile.open(testfile) as ff:
+            pass
+
+    assert len(warnings) == 1, display_warnings(warnings)
+    assert ("was created with extension 'foo.bar.FooBar', which is not "
+        "currently installed (from package foo-1.2.3)") in str(warnings[0].message)
+
+    # Make sure that suppressing the warning works as well
+    with pytest.warns(None) as warnings:
+        with asdf.AsdfFile.open(testfile, ignore_missing_extensions=True) as ff:
+            pass
+
+    assert len(warnings) == 0, display_warnings(warnings)
+
+    with pytest.raises(RuntimeError):
+        with asdf.AsdfFile.open(testfile, strict_extension_check=True) as ff:
+            pass
