@@ -1,8 +1,6 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
 # -*- coding: utf-8 -*-
 
-from __future__ import absolute_import, division, unicode_literals, print_function
-
 import io
 import os
 import re
@@ -10,11 +8,11 @@ import warnings
 
 from jsonschema import ValidationError
 
-import six
 import yaml
 import pytest
 
 import numpy as np
+from numpy.testing import assert_array_equal
 
 import asdf
 from asdf import asdftypes
@@ -22,16 +20,14 @@ from asdf import extension
 from asdf import resolver
 from asdf import schema
 from asdf import util
+from asdf import yamlutil
 
 from asdf.tests import helpers
-from astropy.tests.helper import catch_warnings
-
-
-TEST_DATA_PATH = os.path.join(os.path.dirname(__file__), 'data')
 
 
 class CustomExtension:
-    """This is the base class that is used for extensions for custom tag
+    """
+    This is the base class that is used for extensions for custom tag
     classes that exist only for the purposes of testing.
     """
     @property
@@ -46,21 +42,26 @@ class CustomExtension:
     @property
     def url_mapping(self):
         return [('http://nowhere.org/schemas/custom/',
-                 util.filepath_to_url(TEST_DATA_PATH) +
+                 util.filepath_to_url(helpers.get_test_data_path('')) +
                  '/{url_suffix}.yaml')]
 
 
-def test_violate_toplevel_schema():
-    tree = {'fits': 'This does not look like a FITS file'}
+class TagReferenceType(asdftypes.CustomType):
+    """
+    This class is used by several tests below for validating foreign type
+    references in schemas and ASDF files.
+    """
+    name = 'tag_reference'
+    organization = 'nowhere.org'
+    version = (1, 0, 0)
+    standard = 'custom'
 
-    with pytest.raises(ValidationError):
-        asdf.AsdfFile(tree)
-
-    ff = asdf.AsdfFile()
-    ff.tree['fits'] = 'This does not look like a FITS file'
-    with pytest.raises(ValidationError):
-        buff = io.BytesIO()
-        ff.write_to(buff)
+    @classmethod
+    def from_tree(cls, tree, ctx):
+        node = {}
+        node['name'] = tree['name']
+        node['things'] = yamlutil.tagged_tree_to_custom_tree(tree['things'], ctx)
+        return node
 
 
 @pytest.mark.importorskip('astropy')
@@ -77,7 +78,7 @@ not_unit:
     with asdf.AsdfFile.open(buff) as ff:
         assert isinstance(ff.tree['unit'], u.UnitBase)
         assert not isinstance(ff.tree['not_unit'], u.UnitBase)
-        assert isinstance(ff.tree['not_unit'], six.text_type)
+        assert isinstance(ff.tree['not_unit'], str)
 
         assert ff.tree == {
             'unit': u.m,
@@ -91,7 +92,7 @@ def test_read_json_schema():
     This was known to fail on Python 3.5 See issue #314 at
     https://github.com/spacetelescope/asdf/issues/314 for more details.
     """
-    json_schema = os.path.join(TEST_DATA_PATH, 'example_schema.json')
+    json_schema = helpers.get_test_data_path('example_schema.json')
     schema_tree = schema.load_schema(json_schema, resolve_references=True)
     schema.check_schema(schema_tree)
 
@@ -261,7 +262,7 @@ custom: !<tag:nowhere.org:custom/custom-1.0.0>
     # This should cause a warning but not an error because without explicitly
     # providing an extension, our custom type will not be recognized and will
     # simply be converted to a raw type.
-    with catch_warnings() as warning:
+    with pytest.warns(None) as warning:
         with asdf.AsdfFile.open(buff):
             pass
     assert len(warning) == 1
@@ -372,10 +373,74 @@ custom: !<tag:nowhere.org:custom/default-1.0.0>
         assert 'c' not in ff.tree['custom']['b']
 
 
-def test_references_in_schema():
+def test_tag_reference_validation():
+    class DefaultTypeExtension(CustomExtension):
+        @property
+        def types(self):
+            return [TagReferenceType]
+
+    yaml = """
+custom: !<tag:nowhere.org:custom/tag_reference-1.0.0>
+  name:
+    "Something"
+  things: !core/ndarray-1.0.0
+    data: [1, 2, 3]
+    """
+
+    buff = helpers.yaml_to_asdf(yaml)
+    with asdf.AsdfFile.open(buff, extensions=[DefaultTypeExtension()]) as ff:
+        custom = ff.tree['custom']
+        assert custom['name'] == "Something"
+        assert_array_equal(custom['things'], [1, 2, 3])
+
+
+def test_foreign_tag_reference_validation():
+    class ForeignTagReferenceType(asdftypes.CustomType):
+        name = 'foreign_tag_reference'
+        organization = 'nowhere.org'
+        version = (1, 0, 0)
+        standard = 'custom'
+
+        @classmethod
+        def from_tree(cls, tree, ctx):
+            node = {}
+            node['a'] = yamlutil.tagged_tree_to_custom_tree(tree['a'], ctx)
+            node['b'] = yamlutil.tagged_tree_to_custom_tree(tree['b'], ctx)
+            return node
+
+    class ForeignTypeExtension(CustomExtension):
+        @property
+        def types(self):
+            return [TagReferenceType, ForeignTagReferenceType]
+
+    yaml = """
+custom: !<tag:nowhere.org:custom/foreign_tag_reference-1.0.0>
+  a: !<tag:nowhere.org:custom/tag_reference-1.0.0>
+    name:
+      "Something"
+    things: !core/ndarray-1.0.0
+      data: [1, 2, 3]
+  b: !<tag:nowhere.org:custom/tag_reference-1.0.0>
+    name:
+      "Anything"
+    things: !core/ndarray-1.0.0
+      data: [4, 5, 6]
+    """
+
+    buff = helpers.yaml_to_asdf(yaml)
+    with asdf.AsdfFile.open(buff, extensions=ForeignTypeExtension()) as ff:
+        a = ff.tree['custom']['a']
+        b = ff.tree['custom']['b']
+        assert a['name'] == 'Something'
+        assert_array_equal(a['things'], [1, 2, 3])
+        assert b['name'] == 'Anything'
+        assert_array_equal(b['things'], [4, 5, 6])
+
+
+def test_self_reference_resolution():
     r = resolver.Resolver(CustomExtension().url_mapping, 'url')
     s = schema.load_schema(
-        os.path.join(TEST_DATA_PATH, 'self_referencing-1.0.0.yaml'),
+        helpers.get_test_data_path('self_referencing-1.0.0.yaml'),
         resolver=r,
         resolve_references=True)
     assert '$ref' not in repr(s)
@@ -426,7 +491,7 @@ custom: !<tag:nowhere.org:custom/missing-1.1.0>
   b: {foo: 42}
     """
     buff = helpers.yaml_to_asdf(yaml)
-    with catch_warnings() as w:
+    with pytest.warns(None) as w:
         with asdf.AsdfFile.open(buff, extensions=[DefaultTypeExtension()]) as ff:
             assert ff.tree['custom']['b']['foo'] == 42
 
@@ -458,6 +523,97 @@ def test_assert_roundtrip_with_extension(tmpdir):
     def check(ff):
         assert isinstance(ff.tree['custom'], CustomType)
 
-    helpers.assert_roundtrip_tree(tree, tmpdir, extensions=[CustomTypeExtension()])
+    with pytest.warns(None) as warnings:
+        helpers.assert_roundtrip_tree(
+            tree, tmpdir, extensions=[CustomTypeExtension()])
+
+    assert len(warnings) == 0, helpers.display_warnings(warnings)
 
     assert called_custom_assert_equal[0] is True
+
+
+def test_custom_validation_bad(tmpdir):
+    custom_schema_path = helpers.get_test_data_path('custom_schema.yaml')
+    asdf_file = os.path.join(str(tmpdir), 'out.asdf')
+
+    # This tree does not conform to the custom schema
+    tree = {'stuff': 42, 'other_stuff': 'hello'}
+
+    # Creating file without custom schema should pass
+    with asdf.AsdfFile(tree) as ff:
+        ff.write_to(asdf_file)
+
+    # Creating file using custom schema should fail
+    with pytest.raises(ValidationError):
+        with asdf.AsdfFile(tree, custom_schema=custom_schema_path) as ff:
+            pass
+
+    # Opening file without custom schema should pass
+    with asdf.open(asdf_file) as ff:
+        pass
+
+    # Opening file with custom schema should fail
+    with pytest.raises(ValidationError):
+        with asdf.open(asdf_file, custom_schema=custom_schema_path) as ff:
+            pass
+
+
+def test_custom_validation_good(tmpdir):
+    custom_schema_path = helpers.get_test_data_path('custom_schema.yaml')
+    asdf_file = os.path.join(str(tmpdir), 'out.asdf')
+
+    # This tree conforms to the custom schema
+    tree = {
+        'foo': {'x': 42, 'y': 10},
+        'bar': {'a': 'hello', 'b': 'banjo'}
+    }
+
+    with asdf.AsdfFile(tree, custom_schema=custom_schema_path) as ff:
+        ff.write_to(asdf_file)
+
+    with asdf.open(asdf_file, custom_schema=custom_schema_path) as ff:
+        pass
+
+
+def test_custom_validation_with_definitions_good(tmpdir):
+    custom_schema_path = helpers.get_test_data_path('custom_schema_definitions.yaml')
+    asdf_file = os.path.join(str(tmpdir), 'out.asdf')
+
+    # This tree conforms to the custom schema
+    tree = {
+        'thing': { 'biz': 'hello', 'baz': 'world' }
+    }
+
+    with asdf.AsdfFile(tree, custom_schema=custom_schema_path) as ff:
+        ff.write_to(asdf_file)
+
+    with asdf.open(asdf_file, custom_schema=custom_schema_path) as ff:
+        pass
+
+
+def test_custom_validation_with_definitions_bad(tmpdir):
+    custom_schema_path = helpers.get_test_data_path('custom_schema_definitions.yaml')
+    asdf_file = os.path.join(str(tmpdir), 'out.asdf')
+
+    # This tree does NOT conform to the custom schema
+    tree = {
+        'forb': { 'biz': 'hello', 'baz': 'world' }
+    }
+
+    # Creating file without custom schema should pass
+    with asdf.AsdfFile(tree) as ff:
+        ff.write_to(asdf_file)
+
+    # Creating file with custom schema should fail
+    with pytest.raises(ValidationError):
+        with asdf.AsdfFile(tree, custom_schema=custom_schema_path) as ff:
+            pass
+
+    # Opening file without custom schema should pass
+    with asdf.open(asdf_file) as ff:
+        pass
+
+    # Opening file with custom schema should fail
+    with pytest.raises(ValidationError):
+        with asdf.open(asdf_file, custom_schema=custom_schema_path) as ff:
+            pass
