@@ -1,8 +1,6 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
 # -*- coding: utf-8 -*-
 
-from __future__ import absolute_import, division, unicode_literals, print_function
-
 import io
 from collections import OrderedDict
 
@@ -10,13 +8,11 @@ import numpy as np
 
 import pytest
 
-import six
-
 import yaml
 
-from .. import asdf
-from .. import tagged
-from .. import treeutil
+import asdf
+from asdf import tagged
+from asdf import treeutil
 
 from . import helpers
 
@@ -49,7 +45,8 @@ def test_ordered_dict(tmpdir):
     def check_raw_yaml(content):
         assert b'OrderedDict' not in content
 
-    helpers.assert_roundtrip_tree(tree, tmpdir, check_asdf, check_raw_yaml)
+    helpers.assert_roundtrip_tree(tree, tmpdir, asdf_check_func=check_asdf,
+                                  raw_yaml_check_func=check_raw_yaml)
 
 
 def test_unicode_write(tmpdir):
@@ -63,7 +60,7 @@ def test_unicode_write(tmpdir):
 
     def check_asdf(asdf):
         assert "ɐʇɐp‾ǝpoɔıun" in asdf.tree
-        assert isinstance(asdf.tree['ascii_only'], six.text_type)
+        assert isinstance(asdf.tree['ascii_only'], str)
 
     def check_raw_yaml(content):
         # Ensure that unicode is written out as UTF-8 without escape
@@ -72,7 +69,8 @@ def test_unicode_write(tmpdir):
         # Ensure that the unicode "tag" is not used
         assert b"unicode" not in content
 
-    helpers.assert_roundtrip_tree(tree, tmpdir, check_asdf, check_raw_yaml)
+    helpers.assert_roundtrip_tree(tree, tmpdir, asdf_check_func=check_asdf,
+                                  raw_yaml_check_func=check_raw_yaml)
 
 
 def test_arbitrary_python_object():
@@ -105,7 +103,8 @@ def test_python_tuple(tmpdir):
     def check_raw_yaml(content):
         assert b'tuple' not in content
 
-    helpers.assert_roundtrip_tree(tree, tmpdir, check_asdf, check_raw_yaml)
+    helpers.assert_roundtrip_tree(tree, tmpdir, asdf_check_func=check_asdf,
+                                  raw_yaml_check_func=check_raw_yaml)
 
 
 def test_tags_removed_after_load(tmpdir):
@@ -118,7 +117,7 @@ def test_tags_removed_after_load(tmpdir):
             if node != asdf.tree:
                 assert not isinstance(node, tagged.Tagged)
 
-    helpers.assert_roundtrip_tree(tree, tmpdir, check_asdf)
+    helpers.assert_roundtrip_tree(tree, tmpdir, asdf_check_func=check_asdf)
 
 
 def test_explicit_tags():
@@ -175,3 +174,12 @@ def test_yaml_nan_inf():
         assert np.isnan(ff.tree['a'])
         assert np.isinf(ff.tree['b'])
         assert np.isinf(ff.tree['c'])
+
+
+def test_tag_object():
+    class SomeObject(object):
+        pass
+
+    tag = 'tag:nowhere.org:none/some/thing'
+    instance = tagged.tag_object(tag, SomeObject())
+    assert instance._tag == tag
